@@ -8,12 +8,13 @@ from typing import Any, Dict, List, Optional, Union
 import stringcase
 import yaml
 from datamodel_code_generator import DataModelField, snooper_to_methods
-from datamodel_code_generator.imports import IMPORT_LIST, Import, Imports
-from datamodel_code_generator.model.pydantic.types import type_map
+from datamodel_code_generator.imports import Import, Imports
 from datamodel_code_generator.parser.jsonschema import (
     JsonSchemaObject,
-    json_schema_data_formats,
+    get_model_by_path,
 )
+from datamodel_code_generator.parser.openapi import OpenAPIParser as OpenAPIModelParser
+from datamodel_code_generator.types import DataType
 from pydantic import BaseModel, root_validator
 
 MODEL_PATH = ".models"
@@ -87,6 +88,8 @@ class Operation(CachedPropertyModel):
     requestBody: Dict[str, Any] = {}
     imports: List[Import] = []
     security: Optional[List[Dict[str, List[str]]]] = None
+    components: Dict[str, Any] = {}
+    open_api_model_parser: OpenAPIModelParser
 
     @cached_property
     def root_path(self) -> UsefulStr:
@@ -106,19 +109,17 @@ class Operation(CachedPropertyModel):
             for content_type, schema in requests.contents.items():
                 # TODO: support other content-types
                 if content_type == "application/json":
+                    data_type = self.get_data_type(schema)
                     arguments.append(
                         # TODO: support multiple body
                         Argument(
                             name='body',  # type: ignore
-                            type_hint=schema.ref_object_name,
+                            type_hint=data_type.type_hint,
                             required=requests.required,
                         )
                     )
-                    self.imports.append(
-                        Import(
-                            from_=model_path_var.get(), import_=schema.ref_object_name
-                        )
-                    )
+                    self.imports.extend(data_type.imports_)
+
         if not arguments:
             return None
         return arguments[0]
@@ -144,8 +145,15 @@ class Operation(CachedPropertyModel):
     def response_objects(self) -> List[Response]:
         responses: List[Response] = []
         for status_code, detail in self.responses.items():
+            ref: Optional[str] = detail.get('$ref')
+            if ref and ref.startswith('#/components/'):
+                content: Dict[str, Any] = get_model_by_path(
+                    self.components, ref[13:].split('/')
+                ).get("content", {})
+            else:
+                content = detail.get("content", {})
             contents = {}
-            for content_type, obj in detail.get("content", {}).items():
+            for content_type, obj in content.items():
                 contents[content_type] = (
                     JsonSchemaObject.parse_obj(obj["schema"])
                     if "schema" in obj
@@ -198,20 +206,37 @@ class Operation(CachedPropertyModel):
             arguments.append(self.request)
         return arguments
 
+    def get_data_type(self, schema: JsonSchemaObject) -> DataType:
+        if schema.ref:
+            data_type = self.open_api_model_parser.get_ref_data_type(schema.ref)
+            data_type.imports_.append(
+                Import(
+                    # TODO: Improve import statements
+                    from_=model_path_var.get(),
+                    import_=data_type.type,
+                )
+            )
+            return data_type
+        elif schema.is_array:
+            # TODO: Improve handling array
+            items = schema.items if isinstance(schema.items, list) else [schema.items]
+            return self.open_api_model_parser.data_type(
+                data_types=[self.get_data_type(i) for i in items], is_list=True
+            )
+        return self.open_api_model_parser.get_data_type(schema)
+
     def get_parameter_type(
         self, parameter: Dict[str, Union[str, Dict[str, str]]], snake_case: bool
     ) -> Argument:
-        schema: JsonSchemaObject = JsonSchemaObject.parse_obj(parameter["schema"])
-        format_ = schema.format or "default"
-        type_ = json_schema_data_formats[schema.type][format_]
         name: str = parameter["name"]  # type: ignore
         orig_name = name
         if snake_case:
             name = stringcase.snakecase(name)
+        schema: JsonSchemaObject = JsonSchemaObject.parse_obj(parameter["schema"])
 
         field = DataModelField(
             name=name,
-            data_types=[type_map[type_]],
+            data_type=self.get_data_type(schema),
             required=parameter.get("required") or parameter.get("in") == "path",
         )
         self.imports.extend(field.imports)
@@ -232,46 +257,21 @@ class Operation(CachedPropertyModel):
 
     @cached_property
     def response(self) -> str:
-        models: List[str] = []
+        data_types: List[DataType] = []
         for response in self.response_objects:
             # expect 2xx
             if response.status_code.startswith("2"):
                 for content_type, schema in response.contents.items():
                     if content_type == "application/json":
-                        if schema.is_array:
-                            if isinstance(schema.items, list):
-                                type_ = f'List[{",".join(i.ref_object_name for i in schema.items)}]'
-                                self.imports.extend(
-                                    Import(
-                                        from_=model_path_var.get(),
-                                        import_=i.ref_object_name,
-                                    )
-                                    for i in schema.items
-                                )
-                            else:
-                                type_ = f'List[{schema.items.ref_object_name}]'
-                                self.imports.append(
-                                    Import(
-                                        from_=model_path_var.get(),
-                                        import_=schema.items.ref_object_name,
-                                    )
-                                )
-                            self.imports.append(IMPORT_LIST)
-                        else:
-                            type_ = schema.ref_object_name
-                            self.imports.append(
-                                Import(
-                                    from_=model_path_var.get(),
-                                    import_=schema.ref_object_name,
-                                )
-                            )
-                        models.append(type_)
+                        data_type = self.get_data_type(schema)
+                        data_types.append(data_type)
+                        self.imports.extend(data_type.imports_)
 
-        if not models:
+        if not data_types:
             return "None"
-        if len(models) > 1:
-            return f'Union[{",".join(models)}]'
-        return models[0]
+        if len(data_types) > 1:
+            return self.open_api_model_parser.data_type(data_types=data_types).type_hint
+        return data_types[0].type_hint
 
 
 OPERATION_NAMES: List[str] = [
@@ -287,6 +287,9 @@ OPERATION_NAMES: List[str] = [
 
 
 class Operations(BaseModel):
+    class Config:
+        arbitrary_types_allowed = (OpenAPIModelParser,)
+
     parameters: List[Dict[str, Any]] = []
     get: Optional[Operation] = None
     put: Optional[Operation] = None
@@ -298,19 +301,30 @@ class Operations(BaseModel):
     trace: Optional[Operation] = None
     path: UsefulStr
     security: Optional[List[Dict[str, List[str]]]] = []
+    components: Dict[str, Any] = {}
+    open_api_model_parser: OpenAPIModelParser
 
     @root_validator(pre=True)
     def inject_path_and_type_to_operation(cls, values: Dict[str, Any]) -> Any:
         path: Any = values.get('path')
+        open_api_model_parser: OpenAPIModelParser = values.get('open_api_model_parser')
         return dict(
             **{
-                o: dict(**v, path=path, type=o)
+                o: dict(
+                    **v,
+                    path=path,
+                    type=o,
+                    components=values.get('components', {}),
+                    open_api_model_parser=open_api_model_parser,
+                )
                 for o in OPERATION_NAMES
                 if (v := values.get(o))
             },
             path=path,
             parameters=values.get('parameters', []),
             security=values.get('security'),
+            components=values.get('components', {}),
+            open_api_model_parser=open_api_model_parser,
         )
 
     @root_validator
@@ -330,6 +344,8 @@ class Path(CachedPropertyModel):
     path: UsefulStr
     operations: Optional[Operations] = None
     security: Optional[List[Dict[str, List[str]]]] = []
+    components: Dict[str, Any] = {}
+    open_api_model_parser: OpenAPIModelParser
 
     @root_validator(pre=True)
     def validate_root(cls, values: Dict[str, Any]) -> Any:
@@ -338,12 +354,20 @@ class Path(CachedPropertyModel):
                 if operations := values.get('operations'):
                     if isinstance(operations, dict):
                         security = values.get('security', [])
+                        components = values.get('components', {})
+                        open_api_model_parser = values.get('open_api_model_parser')
                         return {
                             'path': path,
                             'operations': dict(
-                                **operations, path=path, security=security
+                                **operations,
+                                path=path,
+                                security=security,
+                                components=components,
+                                open_api_model_parser=open_api_model_parser,
                             ),
                             'security': security,
+                            'components': components,
+                            'open_api_model_parser': open_api_model_parser,
                         }
         return values
 
@@ -362,11 +386,16 @@ Path.update_forward_refs()
 
 
 class ParsedObject:
-    def __init__(self, parsed_operations: List[Operation]):
+    def __init__(
+        self,
+        parsed_operations: List[Operation],
+        info: Optional[List[Dict[str, Any]]] = None,
+    ):
         self.operations: List[Operation] = sorted(
             parsed_operations, key=lambda m: m.path
         )
         self.imports: Imports = Imports()
+        self.info = info
         for operation in self.operations:
             # create imports
             operation.arguments
@@ -385,6 +414,7 @@ class OpenAPIParser:
         self.input_text: str = input_text
         if model_path:
             model_path_var.set(model_path)
+        self.open_api_model_parser: OpenAPIModelParser = OpenAPIModelParser(source='')
 
     def parse(self) -> ParsedObject:
         openapi = yaml.safe_load(self.input_text)
@@ -395,14 +425,25 @@ class OpenAPIParser:
     ) -> Optional[List[Dict[str, List[str]]]]:
         return openapi.get('security')
 
+    def parse_info(
+        self, openapi: Dict[str, Any]
+    ) -> Optional[List[Dict[str, List[str]]]]:
+        return openapi.get('info')
+
     def parse_paths(self, openapi: Dict[str, Any]) -> ParsedObject:
         security = self.parse_security(openapi)
+        info = self.parse_info(openapi)
         return ParsedObject(
             [
                 operation
                 for path_name, operations in openapi['paths'].items()
                 for operation in Path(
-                    path=UsefulStr(path_name), operations=operations, security=security
+                    path=UsefulStr(path_name),
+                    operations=operations,
+                    security=security,
+                    components=openapi.get('components', {}),
+                    open_api_model_parser=self.open_api_model_parser,
                 ).exists_operations
-            ]
+            ],
+            info,
         )
